from datetime import datetime
from django.core.exceptions import ImproperlyConfigured
from django.utils import timezone
from . import const


def affiliate_middleware(get_response):
    """
    Sets `request.affiliate_id` and `request.affiliate_date` in the session and on the request object
    if the `PARAM_NAME` is found in the request or if an affiliate already existed in the session.
    """
    # very loosely inspired by https://github.com/st4lk/django-affiliate

    def middleware(request):
        session = getattr(request, 'session', None)
        if not session:
            raise ImproperlyConfigured(
                "session attribute should be set for request. Please add "
                "'django.contrib.sessions.middleware.SessionMiddleware' "
                "to your MIDDLEWARE_CLASSES")

        now = timezone.now()
        affiliate_id = request.GET.get(const.AFFILIATE_PARAM_NAME, None)
        if affiliate_id:
            print('setting affiliate id!')
            session[const.AFFILIATE_SESSION_KEY] = affiliate_id
            session[const.AFFILIATE_DATE_SESSION_KEY] = now.isoformat()
        else:
            print('no affiliate id!')
        request.affiliate_id = session.get(const.AFFILIATE_SESSION_KEY, None)
        unparsed_date = session.get(const.AFFILIATE_DATE_SESSION_KEY, None)
        try:
            request.affiliate_date = datetime.strptime(unparsed_date, "%Y-%m-%dT%H:%M:%S.%fZ") if unparsed_date else None
        except ValueError:
            request.affiliate_date = None

        return get_response(request)

    return middleware



from setuptools import setup, find_packages
from codecs import open
from os import path

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='django-simple-affiliate',
    version='0.1',
    description='Simple Affiliate System for Django',
    long_description=long_description,
    url='https://github.com/czue/django-simple-affiliate',
    author='Cory Zue',
    author_email='cory@coryzue.com',
    license='Apache Software License',

    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Libraries',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],

    keywords='affiliate marketing django middleware tracking',
    packages=find_packages(),
    install_requires=['django'],

    extras_require={},
    package_data={},
    data_files=[],
    entry_points={},
)



from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from django.utils import timezone


def get_affiliate_id(request, expiry_period=None):
    expiry_period = expiry_period or getattr(settings, 'AFFILIATE_EXPIRY_PERIOD', None)
    try:
        affiliate_id = request.affiliate_id
    except AttributeError:
        raise ImproperlyConfigured('You need to add "simple_affiliate.middleware.affiliate_middleware" to '
                                   'your MIDDLEWARE in settings.py to use this function!')

    if expiry_period and affiliate_id and timezone.now() - expiry_period > request.affiliate_date:
        # a valid affiliate ID has expired
        return None
    else:
        return affiliate_id
